// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en
import { randomBytes } from 'crypto';
import ByteSerializer from './byteSerializer';
import AESCipher from './aesCipher';
import Hash from './hash';

export default class AESKey {
    /**
     * @param {Uint8Array} key
     * @param {Uint8Array} mac
     */
    constructor(key = randomBytes(16), mac = randomBytes(32)) {
        this.key = key;
        this.mac = mac;
    }

    /** @param {string | Uint8Array} message */
    hash(message) { return Hash.hmacBuffer(message, this.mac); }

    /** @param {string | Uint8Array} message */
    encrypt(message) { return AESCipher.encrypt(this, message); }

    /** @param {string | Uint8Array} message */
    encryptStr(message) { return AESCipher.encryptStr(this, message); }

    /** @param {string | Uint8Array} message */
    decrypt(message) { return AESCipher.decrypt(this, message); }

    /** @param {string | Uint8Array} message */
    decryptStr(message) { return AESCipher.decryptStr(this, message); }

    /** @param {Uint8Array} data */
    derive(data) {
        return derive(this.mac, data);
    }

    toArray() {
        return ByteSerializer.Serialize([this.key, this.mac], 1);
    }
    
    toString() {
        return Buffer.from(this.toArray()).toString('base64');
    }

    /** @param {string | Uint8Array} key */
    static from(key) {
        var bytes = typeof key == 'string' ? Buffer.from(key, 'base64') : Buffer.from(key);
        var secrets = ByteSerializer.deserializeBytes(bytes, 1);        
        return new AESKey(secrets[0], secrets[1])
    }

    //TODO: Search a better implementation
    /** @param {Uint8Array} secret */
    static seed(secret) {
        return derive(secret);
    }
}

//TODO: Search a better implementation
/** @param {Uint8Array} secret */
function derive(secret, iv = Buffer.alloc(15)) {
    var sec = Hash.hmacBuffer(Buffer.concat([Buffer.from([1]), iv]), secret);
    var mac = Hash.hmacBuffer(Buffer.concat([Buffer.from([2]), iv]), secret);

    return new AESKey(sec.slice(0, 16), mac);
}