// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en

/** @typedef {Uint8Array | string | number | BigInt.BigInteger| BN } BNInput */
import BigInt from 'big-integer';
import BN from 'bn.js';

/** @param {BNInput} number
 *  @returns {BN} */
export function getBN(number) {
    if (number instanceof Uint8Array || typeof number == "string" || typeof number == "number")
        return new BN(number)

    if (number instanceof BN)
        return number;

    if (number != null)
        return new BN(number.toArray(256).value);

    // @ts-ignore
    return number;
};

/** @param {BNInput} number */
export function getBig(number) {
    if (number instanceof BN)
        return BigInt.fromArray(number.toArray('be'), 256, false);
    else if (typeof number === "number")
        return BigInt(number);
    else if (typeof number === "string")
        return BigInt(number);
    else if (number instanceof Uint8Array )
        return BigInt.fromArray(Array.from(number), 256, false);
    else
        return number;
};

/** @param {BigInt.BigInteger|BN} number */
export function getArray(number) {
    if (number instanceof BN)
        return Buffer.from(number.toArray('be'));
    else
        return Buffer.from(number.toArray(256).value);
}